using System;
using System.Collections.Generic;
using System.Text;
using VVVV.PluginInterfaces.V1;
using System.Web.Services.Protocols;
using System.Net;

namespace VVVV.Nodes
{
    /// <summary>
    /// Template for all plugins which require access to a Web Service.<br />
    /// Contains all the methods to handle Urls, Proxies and Credentials
    /// </summary>
    /// <typeparam name="T">Web Service Client</typeparam>
    public abstract class BaseSoapClientPlugin<T> where T : SoapHttpClientProtocol
    {
        #region Protected Members
        /// <summary>
        /// The Plugin Host.
        /// </summary>
        /// <remarks>The Host is automatically attached, so any derived class 
        /// can access it from here.</remarks>
        protected IPluginHost FHost;

        /// <summary>
        /// This is the Service Proxy (Normally generated by adding a Web Reference to a project).
        /// </summary>
        protected T FServiceProxy;
        #endregion

        private bool FUseProxy = false;
        private WebProxy FProxy;

        #region Pins
        private IStringIn FPinInUrl;
        private IValueConfig FPinCfgProxy;

        private IStringIn FPinInProxyUrl;
        private IValueIn FPinInProxyPort;
        private IStringIn FPinInProxyUserName;
        private IStringIn FPinInProxyPassword;
        #endregion

        #region Abstract Methods
        public virtual bool AutoEvaluate
        {
            get { return false; }
        }

        protected abstract void OnConfigurate(IPluginConfig Input);
        protected abstract void OnEvaluate(int SpreadMax, bool settingschanged);
        protected abstract void OnPluginHostSet();
        protected abstract string DefaultUrl { get; }
        #endregion

        #region Configurate
        public void Configurate(IPluginConfig Input)
        {
            double useproxy;
            this.FPinCfgProxy.GetValue(0, out useproxy);
            this.FUseProxy = useproxy >= 0.5;

            if (this.FUseProxy)
            {
                #region If we use a proxy
                this.FHost.CreateStringInput("Proxy URL", TSliceMode.Single, TPinVisibility.True, out this.FPinInProxyUrl);
                this.FPinInProxyUrl.SetSubType("", false);
  
                this.FHost.CreateValueInput("Proxy Port", 1, null, TSliceMode.Single, TPinVisibility.True, out this.FPinInProxyPort);
                this.FPinInProxyPort.SetSubType(0, 65536, 1, 8080, false, false, true);
        
                this.FHost.CreateStringInput("Proxy User Name", TSliceMode.Single, TPinVisibility.True, out this.FPinInProxyUserName);
                this.FPinInProxyUserName.SetSubType("", false);

                this.FHost.CreateStringInput("Proxy Password", TSliceMode.Single, TPinVisibility.True, out this.FPinInProxyPassword);
                this.FPinInProxyPassword.SetSubType("", false);

                this.FServiceProxy.Proxy = new WebProxy();
                #endregion
            }
            else
            {
                #region No Proxy Used
                if (this.FPinInProxyUrl != null) 
                {
                    this.FHost.DeletePin(this.FPinInProxyUrl);
                    this.FHost.DeletePin(this.FPinInProxyPassword);
                    this.FHost.DeletePin(this.FPinInProxyUserName);
                    this.FHost.DeletePin(this.FPinInProxyPort);

                    this.FPinInProxyUrl = null;
                    this.FPinInProxyUserName = null;
                    this.FPinInProxyPassword = null;
                    this.FPinInProxyPort = null;

                    this.FServiceProxy.Proxy = null;
                }
                #endregion
            }

            this.OnConfigurate(Input);
        }
        #endregion

        #region Evaluate
        public void Evaluate(int SpreadMax)
        {
            bool settingschanged = false;

            #region Update Url
            if (this.FPinInUrl.PinIsChanged)
            {
                string url;
                this.FPinInUrl.GetString(0, out url);
                this.FServiceProxy.Url = url;
                settingschanged = true;
            }
            #endregion

            #region Reconfigure Proxy Settings
            if (this.FUseProxy)
            {
                if (this.FPinInProxyPassword.PinIsChanged || this.FPinInProxyPort.PinIsChanged || this.FPinInProxyUserName.PinIsChanged || this.FPinInProxyUrl.PinIsChanged)
                {
                    string purl, puser, ppwd;
                    double pport;

                    this.FPinInProxyUrl.GetString(0, out purl);
                    this.FPinInProxyPort.GetValue(0, out pport);
                    this.FPinInProxyUserName.GetString(0, out puser);
                    this.FPinInProxyPassword.GetString(0, out ppwd);

                    string uri = purl + ":" + pport;
                    this.FProxy = new WebProxy(purl, Convert.ToInt32(pport));

                    if (puser.Trim().Length > 0 && ppwd.Trim().Length > 0)
                    {
                        this.FProxy.Credentials = new NetworkCredential(puser, ppwd);
                    }

                    settingschanged = true;
                }
            }
            #endregion

            this.OnEvaluate(SpreadMax, settingschanged);
        }
        #endregion

        #region Set Plugin Host
        public void SetPluginHost(IPluginHost Host)
        {
            this.FHost = Host;

            try
            {
                this.FServiceProxy = (T)Activator.CreateInstance(typeof(T));
            }
            catch (Exception ex)
            {
                this.FHost.Log(TLogType.Error, ex.Message);
            }

            this.FHost.CreateStringInput("URL", TSliceMode.Single, TPinVisibility.True, out this.FPinInUrl);
            this.FPinInUrl.SetSubType(this.DefaultUrl, false);
  
            this.FHost.CreateValueConfig("Use Proxy", 1, null, TSliceMode.Single, TPinVisibility.True, out this.FPinCfgProxy);
            this.FPinCfgProxy.SetSubType(0, 1, 1, 0, false, true, true);

            this.OnPluginHostSet();
        }
        #endregion
    }
}
